@@ -21,12 +21,13 @@ package admitters
 
 import (
 	"encoding/json"
+	"fmt"
 
 	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
 
-	"k8s.io/api/admission/v1beta1"
+	admissionv1 "k8s.io/api/admission/v1"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -34,8 +35,8 @@ import (
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
 
-	v1 "kubevirt.io/client-go/api/v1"
-	snapshotv1 "kubevirt.io/client-go/apis/snapshot/v1alpha1"
+	v1 "kubevirt.io/api/core/v1"
+	snapshotv1 "kubevirt.io/api/snapshot/v1alpha1"
 	kubevirtfake "kubevirt.io/client-go/generated/kubevirt/clientset/versioned/fake"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/kubevirt/pkg/testutils"
@@ -54,6 +55,7 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 
 	t := true
 	f := false
+	runStrategyManual := v1.RunStrategyManual
 
 	snapshot := &snapshotv1.VirtualMachineSnapshot{
 		ObjectMeta: metav1.ObjectMeta{
@@ -66,7 +68,7 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 		},
 	}
 
-	config, configMapInformer, _, _ := testutils.NewFakeClusterConfig(&corev1.ConfigMap{})
+	config, _, kvInformer := testutils.NewFakeClusterConfigUsingKVConfig(&v1.KubeVirtConfiguration{})
 
 	Context("Without feature gate enabled", func() {
 		It("should reject anything", func() {
@@ -87,13 +89,26 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 
 	Context("With feature gate enabled", func() {
 		enableFeatureGate := func(featureGate string) {
-			testutils.UpdateFakeClusterConfig(configMapInformer, &corev1.ConfigMap{
-				Data: map[string]string{virtconfig.FeatureGatesKey: featureGate},
+			testutils.UpdateFakeKubeVirtClusterConfig(kvInformer, &v1.KubeVirt{
+				Spec: v1.KubeVirtSpec{
+					Configuration: v1.KubeVirtConfiguration{
+						DeveloperConfiguration: &v1.DeveloperConfiguration{
+							FeatureGates: []string{featureGate},
+						},
+					},
+				},
 			})
 		}
-
 		disableFeatureGates := func() {
-			testutils.UpdateFakeClusterConfig(configMapInformer, &corev1.ConfigMap{})
+			testutils.UpdateFakeKubeVirtClusterConfig(kvInformer, &v1.KubeVirt{
+				Spec: v1.KubeVirtSpec{
+					Configuration: v1.KubeVirtConfiguration{
+						DeveloperConfiguration: &v1.DeveloperConfiguration{
+							FeatureGates: make([]string, 0),
+						},
+					},
+				},
+			})
 		}
 
 		BeforeEach(func() {
@@ -105,8 +120,8 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 		})
 
 		It("should reject invalid request resource", func() {
-			ar := &v1beta1.AdmissionReview{
-				Request: &v1beta1.AdmissionRequest{
+			ar := &admissionv1.AdmissionReview{
+				Request: &admissionv1.AdmissionRequest{
 					Resource: webhooks.VirtualMachineGroupVersionResource,
 				},
 			}
@@ -158,7 +173,7 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 			resp := createTestVMRestoreAdmitter(config, nil, snapshot).Admit(ar)
 			Expect(resp.Allowed).To(BeFalse())
 			Expect(len(resp.Result.Details.Causes)).To(Equal(1))
-			Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target.name"))
+			Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target"))
 		})
 
 		It("should reject when VM and snapshot do not exist", func() {
@@ -181,7 +196,7 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 			resp := createTestVMRestoreAdmitter(config, nil).Admit(ar)
 			Expect(resp.Allowed).To(BeFalse())
 			Expect(len(resp.Result.Details.Causes)).To(Equal(2))
-			Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target.name"))
+			Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target"))
 			Expect(resp.Result.Details.Causes[1].Field).To(Equal("spec.virtualMachineSnapshotName"))
 		})
 
@@ -292,7 +307,33 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 				resp := createTestVMRestoreAdmitter(config, vm, snapshot).Admit(ar)
 				Expect(resp.Allowed).To(BeFalse())
 				Expect(len(resp.Result.Details.Causes)).To(Equal(1))
-				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target.name"))
+				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target"))
+			})
+
+			It("should reject when VM run strategy is not halted", func() {
+				restore := &snapshotv1.VirtualMachineRestore{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      "restore",
+						Namespace: "default",
+					},
+					Spec: snapshotv1.VirtualMachineRestoreSpec{
+						Target: corev1.TypedLocalObjectReference{
+							APIGroup: &apiGroup,
+							Kind:     "VirtualMachine",
+							Name:     vmName,
+						},
+						VirtualMachineSnapshotName: vmSnapshotName,
+					},
+				}
+
+				vm.Spec.RunStrategy = &runStrategyManual
+
+				ar := createRestoreAdmissionReview(restore)
+				resp := createTestVMRestoreAdmitter(config, vm, snapshot).Admit(ar)
+				Expect(resp.Allowed).To(BeFalse())
+				Expect(len(resp.Result.Details.Causes)).To(Equal(1))
+				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target"))
+				Expect(resp.Result.Details.Causes[0].Message).To(Equal(fmt.Sprintf("VirtualMachine %q run strategy has to be %s", vmName, v1.RunStrategyHalted)))
 			})
 
 			It("should reject when snapshot does not exist", func() {
@@ -320,6 +361,33 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.virtualMachineSnapshotName"))
 			})
 
+			It("should reject when snapshot has failed", func() {
+				restore := &snapshotv1.VirtualMachineRestore{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      "restore",
+						Namespace: "default",
+					},
+					Spec: snapshotv1.VirtualMachineRestoreSpec{
+						Target: corev1.TypedLocalObjectReference{
+							APIGroup: &apiGroup,
+							Kind:     "VirtualMachine",
+							Name:     vmName,
+						},
+						VirtualMachineSnapshotName: vmSnapshotName,
+					},
+				}
+
+				vm.Spec.Running = &f
+				s := snapshot.DeepCopy()
+				s.Status.Phase = snapshotv1.Failed
+
+				ar := createRestoreAdmissionReview(restore)
+				resp := createTestVMRestoreAdmitter(config, vm, s).Admit(ar)
+				Expect(resp.Allowed).To(BeFalse())
+				Expect(len(resp.Result.Details.Causes)).To(Equal(1))
+				Expect(resp.Result.Details.Causes[0].Message).To(Equal(fmt.Sprintf("VirtualMachineSnapshot %q has failed and is invalid to use", vmSnapshotName)))
+			})
+
 			It("should reject when snapshot not ready", func() {
 				restore := &snapshotv1.VirtualMachineRestore{
 					ObjectMeta: metav1.ObjectMeta{
@@ -461,7 +529,7 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 				resp := createTestVMRestoreAdmitter(config, vm, snapshot, restoreInProcess).Admit(ar)
 				Expect(resp.Allowed).To(BeFalse())
 				Expect(len(resp.Result.Details.Causes)).To(Equal(1))
-				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target.name"))
+				Expect(resp.Result.Details.Causes[0].Field).To(Equal("spec.target"))
 			})
 
 			It("should accept when VM is not running", func() {
@@ -487,12 +555,12 @@ var _ = Describe("Validating VirtualMachineRestore Admitter", func() {
 	})
 })
 
-func createRestoreAdmissionReview(restore *snapshotv1.VirtualMachineRestore) *v1beta1.AdmissionReview {
+func createRestoreAdmissionReview(restore *snapshotv1.VirtualMachineRestore) *admissionv1.AdmissionReview {
 	bytes, _ := json.Marshal(restore)
 
-	ar := &v1beta1.AdmissionReview{
-		Request: &v1beta1.AdmissionRequest{
-			Operation: v1beta1.Create,
+	ar := &admissionv1.AdmissionReview{
+		Request: &admissionv1.AdmissionRequest{
+			Operation: admissionv1.Create,
 			Namespace: "default",
 			Resource: metav1.GroupVersionResource{
 				Group:    "snapshot.kubevirt.io",
@@ -507,13 +575,13 @@ func createRestoreAdmissionReview(restore *snapshotv1.VirtualMachineRestore) *v1
 	return ar
 }
 
-func createRestoreUpdateAdmissionReview(old, current *snapshotv1.VirtualMachineRestore) *v1beta1.AdmissionReview {
+func createRestoreUpdateAdmissionReview(old, current *snapshotv1.VirtualMachineRestore) *admissionv1.AdmissionReview {
 	oldBytes, _ := json.Marshal(old)
 	currentBytes, _ := json.Marshal(current)
 
-	ar := &v1beta1.AdmissionReview{
-		Request: &v1beta1.AdmissionRequest{
-			Operation: v1beta1.Update,
+	ar := &admissionv1.AdmissionReview{
+		Request: &admissionv1.AdmissionRequest{
+			Operation: admissionv1.Update,
 			Namespace: "default",
 			Resource: metav1.GroupVersionResource{
 				Group:    "snapshot.kubevirt.io",
@@ -546,7 +614,6 @@ func createTestVMRestoreAdmitter(
 	virtClient.EXPECT().VirtualMachine(gomock.Any()).Return(vmInterface).AnyTimes()
 
 	restoreInformer, _ := testutils.NewFakeInformerFor(&snapshotv1.VirtualMachineRestore{})
-	webhooks.GetInformers().VMRestoreInformer = restoreInformer
 	for _, obj := range objs {
 		r, ok := obj.(*snapshotv1.VirtualMachineRestore)
 		if ok {
@@ -560,5 +627,5 @@ func createTestVMRestoreAdmitter(
 	} else {
 		vmInterface.EXPECT().Get(vm.Name, gomock.Any()).Return(vm, nil)
 	}
-	return &VMRestoreAdmitter{Config: config, Client: virtClient}
+	return &VMRestoreAdmitter{Config: config, Client: virtClient, VMRestoreInformer: restoreInformer}
 }
