@@ -25,22 +25,29 @@ import (
 	"time"
 
 	vsv1beta1 "github.com/kubernetes-csi/external-snapshotter/v2/pkg/apis/volumesnapshot/v1beta1"
-	extv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	corev1 "k8s.io/api/core/v1"
+	extv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/util/workqueue"
 
-	kubevirtv1 "kubevirt.io/client-go/api/v1"
-	snapshotv1 "kubevirt.io/client-go/apis/snapshot/v1alpha1"
+	kubevirtv1 "kubevirt.io/api/core/v1"
+	snapshotv1 "kubevirt.io/api/snapshot/v1alpha1"
 	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1alpha1"
+	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/kubevirt/pkg/controller"
 	"kubevirt.io/kubevirt/pkg/util/status"
 )
 
+const (
+	unexpectedResourceFmt  = "unexpected resource %+v"
+	failedKeyFromObjectFmt = "failed to get key from object: %v, %v"
+	enqueuedForSyncFmt     = "enqueued %q for sync"
+)
+
 const (
 	volumeSnapshotCRD      = "volumesnapshots.snapshot.storage.k8s.io"
 	volumeSnapshotClassCRD = "volumesnapshotclasses.snapshot.storage.k8s.io"
@@ -69,6 +76,7 @@ type VMSnapshotController struct {
 	CRDInformer               cache.SharedIndexInformer
 	PodInformer               cache.SharedIndexInformer
 	DVInformer                cache.SharedIndexInformer
+	CRInformer                cache.SharedIndexInformer
 
 	Recorder record.EventRecorder
 
@@ -79,7 +87,6 @@ type VMSnapshotController struct {
 	crdQueue               workqueue.RateLimitingInterface
 	vmSnapshotStatusQueue  workqueue.RateLimitingInterface
 	vmQueue                workqueue.RateLimitingInterface
-	dvQueue                workqueue.RateLimitingInterface
 
 	dynamicInformerMap map[string]*dynamicInformer
 	eventHandlerMap    map[string]cache.ResourceEventHandlerFuncs
@@ -91,25 +98,24 @@ var supportedCRDVersions = []string{"v1beta1"}
 
 // Init initializes the snapshot controller
 func (ctrl *VMSnapshotController) Init() {
-	ctrl.vmSnapshotQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-vmsnapshot")
-	ctrl.vmSnapshotContentQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-vmsnapshotcontent")
-	ctrl.crdQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-crd")
-	ctrl.vmSnapshotStatusQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-vmsnashotstatus")
-	ctrl.vmQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-vm")
-	ctrl.dvQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "snapshot-controller-dv")
+	ctrl.vmSnapshotQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-snapshot-vmsnapshot")
+	ctrl.vmSnapshotContentQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-snapshot-vmsnapshotcontent")
+	ctrl.crdQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-snapshot-crd")
+	ctrl.vmSnapshotStatusQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-snapshot-vmsnashotstatus")
+	ctrl.vmQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-snapshot-vm")
 
 	ctrl.dynamicInformerMap = map[string]*dynamicInformer{
-		volumeSnapshotCRD:      &dynamicInformer{informerFunc: controller.VolumeSnapshotInformer},
-		volumeSnapshotClassCRD: &dynamicInformer{informerFunc: controller.VolumeSnapshotClassInformer},
+		volumeSnapshotCRD:      {informerFunc: controller.VolumeSnapshotInformer},
+		volumeSnapshotClassCRD: {informerFunc: controller.VolumeSnapshotClassInformer},
 	}
 
 	ctrl.eventHandlerMap = map[string]cache.ResourceEventHandlerFuncs{
-		volumeSnapshotCRD: cache.ResourceEventHandlerFuncs{
+		volumeSnapshotCRD: {
 			AddFunc:    ctrl.handleVolumeSnapshot,
 			UpdateFunc: func(oldObj, newObj interface{}) { ctrl.handleVolumeSnapshot(newObj) },
 			DeleteFunc: ctrl.handleVolumeSnapshot,
 		},
-		volumeSnapshotClassCRD: cache.ResourceEventHandlerFuncs{
+		volumeSnapshotClassCRD: {
 			AddFunc:    ctrl.handleVolumeSnapshotClass,
 			UpdateFunc: func(oldObj, newObj interface{}) { ctrl.handleVolumeSnapshotClass(newObj) },
 			DeleteFunc: ctrl.handleVolumeSnapshotClass,
@@ -140,6 +146,14 @@ func (ctrl *VMSnapshotController) Init() {
 		ctrl.ResyncPeriod,
 	)
 
+	ctrl.VMIInformer.AddEventHandlerWithResyncPeriod(
+		cache.ResourceEventHandlerFuncs{
+			AddFunc:    ctrl.handleVMI,
+			UpdateFunc: func(oldObj, newObj interface{}) { ctrl.handleVMI(newObj) },
+		},
+		ctrl.ResyncPeriod,
+	)
+
 	ctrl.CRDInformer.AddEventHandlerWithResyncPeriod(
 		cache.ResourceEventHandlerFuncs{
 			AddFunc:    ctrl.handleCRD,
@@ -158,6 +172,15 @@ func (ctrl *VMSnapshotController) Init() {
 		ctrl.ResyncPeriod,
 	)
 
+	ctrl.PVCInformer.AddEventHandlerWithResyncPeriod(
+		cache.ResourceEventHandlerFuncs{
+			AddFunc:    ctrl.handlePVC,
+			UpdateFunc: func(oldObj, newObj interface{}) { ctrl.handlePVC(newObj) },
+			DeleteFunc: ctrl.handlePVC,
+		},
+		ctrl.ResyncPeriod,
+	)
+
 	ctrl.vmStatusUpdater = status.NewVMStatusUpdater(ctrl.Client)
 }
 
@@ -194,7 +217,6 @@ func (ctrl *VMSnapshotController) Run(threadiness int, stopCh <-chan struct{}) e
 		go wait.Until(ctrl.crdWorker, time.Second, stopCh)
 		go wait.Until(ctrl.vmSnapshotStatusWorker, time.Second, stopCh)
 		go wait.Until(ctrl.vmWorker, time.Second, stopCh)
-		go wait.Until(ctrl.dvWorker, time.Second, stopCh)
 	}
 
 	<-stopCh
@@ -231,11 +253,6 @@ func (ctrl *VMSnapshotController) vmWorker() {
 	}
 }
 
-func (ctrl *VMSnapshotController) dvWorker() {
-	for ctrl.processDVWorkItem() {
-	}
-}
-
 func (ctrl *VMSnapshotController) processVMSnapshotWorkItem() bool {
 	return processWorkItem(ctrl.vmSnapshotQueue, func(key string) (time.Duration, error) {
 		log.Log.V(3).Infof("vmSnapshot worker processing key [%s]", key)
@@ -247,7 +264,7 @@ func (ctrl *VMSnapshotController) processVMSnapshotWorkItem() bool {
 
 		vmSnapshot, ok := storeObj.(*snapshotv1.VirtualMachineSnapshot)
 		if !ok {
-			return 0, fmt.Errorf("unexpected resource %+v", storeObj)
+			return 0, fmt.Errorf(unexpectedResourceFmt, storeObj)
 		}
 
 		return ctrl.updateVMSnapshot(vmSnapshot.DeepCopy())
@@ -265,7 +282,7 @@ func (ctrl *VMSnapshotController) processVMSnapshotContentWorkItem() bool {
 
 		vmSnapshotContent, ok := storeObj.(*snapshotv1.VirtualMachineSnapshotContent)
 		if !ok {
-			return 0, fmt.Errorf("unexpected resource %+v", storeObj)
+			return 0, fmt.Errorf(unexpectedResourceFmt, storeObj)
 		}
 
 		return ctrl.updateVMSnapshotContent(vmSnapshotContent.DeepCopy())
@@ -290,9 +307,9 @@ func (ctrl *VMSnapshotController) processCRDWorkItem() bool {
 			return ctrl.deleteDynamicInformer(name)
 		}
 
-		crd, ok := storeObj.(*extv1beta1.CustomResourceDefinition)
+		crd, ok := storeObj.(*extv1.CustomResourceDefinition)
 		if !ok {
-			return 0, fmt.Errorf("unexpected resource %+v", storeObj)
+			return 0, fmt.Errorf(unexpectedResourceFmt, storeObj)
 		}
 
 		if crd.DeletionTimestamp != nil {
@@ -315,7 +332,7 @@ func (ctrl *VMSnapshotController) processVMSnapshotStatusWorkItem() bool {
 		if exists {
 			vm, ok := storeObj.(*kubevirtv1.VirtualMachine)
 			if !ok {
-				return 0, fmt.Errorf("unexpected resource %+v", storeObj)
+				return 0, fmt.Errorf(unexpectedResourceFmt, storeObj)
 			}
 
 			if err = ctrl.updateVolumeSnapshotStatuses(vm); err != nil {
@@ -339,7 +356,7 @@ func (ctrl *VMSnapshotController) processVMWorkItem() bool {
 		if exists {
 			vm, ok := storeObj.(*kubevirtv1.VirtualMachine)
 			if !ok {
-				return 0, fmt.Errorf("unexpected resource %+v", storeObj)
+				return 0, fmt.Errorf(unexpectedResourceFmt, storeObj)
 			}
 
 			ctrl.handleVM(vm)
@@ -349,28 +366,6 @@ func (ctrl *VMSnapshotController) processVMWorkItem() bool {
 	})
 }
 
-func (ctrl *VMSnapshotController) processDVWorkItem() bool {
-	return processWorkItem(ctrl.dvQueue, func(key string) (time.Duration, error) {
-		log.Log.V(3).Infof("DV worker processing VM [%s]", key)
-
-		storeObj, exists, err := ctrl.DVInformer.GetStore().GetByKey(key)
-		if err != nil {
-			return 0, err
-		}
-
-		if exists {
-			dv, ok := storeObj.(*cdiv1.DataVolume)
-			if !ok {
-				return 0, fmt.Errorf("unexpected resource %+v", storeObj)
-			}
-
-			ctrl.handleDV(dv)
-		}
-
-		return 0, nil
-	})
-}
-
 func (ctrl *VMSnapshotController) handleVMSnapshot(obj interface{}) {
 	if unknown, ok := obj.(cache.DeletedFinalStateUnknown); ok && unknown.Obj != nil {
 		obj = unknown.Obj
@@ -379,10 +374,10 @@ func (ctrl *VMSnapshotController) handleVMSnapshot(obj interface{}) {
 	if vmSnapshot, ok := obj.(*snapshotv1.VirtualMachineSnapshot); ok {
 		objName, err := cache.DeletionHandlingMetaNamespaceKeyFunc(vmSnapshot)
 		if err != nil {
-			log.Log.Errorf("failed to get key from object: %v, %v", err, vmSnapshot)
+			log.Log.Errorf(failedKeyFromObjectFmt, err, vmSnapshot)
 			return
 		}
-		log.Log.V(3).Infof("enqueued %q for sync", objName)
+		log.Log.V(3).Infof(enqueuedForSyncFmt, objName)
 		ctrl.vmSnapshotQueue.Add(objName)
 	}
 }
@@ -395,7 +390,7 @@ func (ctrl *VMSnapshotController) handleVMSnapshotContent(obj interface{}) {
 	if content, ok := obj.(*snapshotv1.VirtualMachineSnapshotContent); ok {
 		objName, err := cache.DeletionHandlingMetaNamespaceKeyFunc(content)
 		if err != nil {
-			log.Log.Errorf("failed to get key from object: %v, %v", err, content)
+			log.Log.Errorf(failedKeyFromObjectFmt, err, content)
 			return
 		}
 
@@ -405,7 +400,7 @@ func (ctrl *VMSnapshotController) handleVMSnapshotContent(obj interface{}) {
 			ctrl.vmSnapshotQueue.Add(k)
 		}
 
-		log.Log.V(5).Infof("enqueued %q for sync", objName)
+		log.Log.V(5).Infof(enqueuedForSyncFmt, objName)
 		ctrl.vmSnapshotContentQueue.Add(objName)
 	}
 }
@@ -416,7 +411,8 @@ func (ctrl *VMSnapshotController) handleVM(obj interface{}) {
 	}
 
 	if vm, ok := obj.(*kubevirtv1.VirtualMachine); ok {
-		keys, err := ctrl.VMSnapshotInformer.GetIndexer().IndexKeys("vm", vm.Name)
+		k, _ := cache.MetaNamespaceKeyFunc(vm)
+		keys, err := ctrl.VMSnapshotInformer.GetIndexer().IndexKeys("vm", k)
 		if err != nil {
 			utilruntime.HandleError(err)
 			return
@@ -435,6 +431,25 @@ func (ctrl *VMSnapshotController) handleVM(obj interface{}) {
 	}
 }
 
+func (ctrl *VMSnapshotController) handleVMI(obj interface{}) {
+	if unknown, ok := obj.(cache.DeletedFinalStateUnknown); ok && unknown.Obj != nil {
+		obj = unknown.Obj
+	}
+
+	if vmi, ok := obj.(*kubevirtv1.VirtualMachineInstance); ok {
+		k, _ := cache.MetaNamespaceKeyFunc(vmi)
+		keys, err := ctrl.VMSnapshotInformer.GetIndexer().IndexKeys("vm", k)
+		if err != nil {
+			utilruntime.HandleError(err)
+			return
+		}
+
+		for _, k := range keys {
+			ctrl.vmSnapshotQueue.Add(k)
+		}
+	}
+}
+
 func (ctrl *VMSnapshotController) handleVolumeSnapshotClass(obj interface{}) {
 	if unknown, ok := obj.(cache.DeletedFinalStateUnknown); ok && unknown.Obj != nil {
 		obj = unknown.Obj
@@ -452,7 +467,7 @@ func (ctrl *VMSnapshotController) handleCRD(obj interface{}) {
 		obj = unknown.Obj
 	}
 
-	if crd, ok := obj.(*extv1beta1.CustomResourceDefinition); ok {
+	if crd, ok := obj.(*extv1.CustomResourceDefinition); ok {
 		_, ok = ctrl.dynamicInformerMap[crd.Name]
 		if ok {
 			hasSupportedVersion := false
@@ -470,11 +485,11 @@ func (ctrl *VMSnapshotController) handleCRD(obj interface{}) {
 
 			objName, err := cache.DeletionHandlingMetaNamespaceKeyFunc(crd)
 			if err != nil {
-				log.Log.Errorf("failed to get key from object: %v, %v", err, crd)
+				log.Log.Errorf(failedKeyFromObjectFmt, err, crd)
 				return
 			}
 
-			log.Log.V(3).Infof("enqueued %q for sync", objName)
+			log.Log.V(3).Infof(enqueuedForSyncFmt, objName)
 			ctrl.crdQueue.Add(objName)
 		}
 	}
@@ -486,7 +501,8 @@ func (ctrl *VMSnapshotController) handleVolumeSnapshot(obj interface{}) {
 	}
 
 	if volumeSnapshot, ok := obj.(*vsv1beta1.VolumeSnapshot); ok {
-		keys, err := ctrl.VMSnapshotContentInformer.GetIndexer().IndexKeys("volumeSnapshot", volumeSnapshot.Name)
+		k, _ := cache.MetaNamespaceKeyFunc(volumeSnapshot)
+		keys, err := ctrl.VMSnapshotContentInformer.GetIndexer().IndexKeys("volumeSnapshot", k)
 		if err != nil {
 			utilruntime.HandleError(err)
 			return
@@ -504,12 +520,37 @@ func (ctrl *VMSnapshotController) handleDV(obj interface{}) {
 	}
 
 	if dv, ok := obj.(*cdiv1.DataVolume); ok {
-		log.Log.V(3).Infof("Processing DV %s/%s", dv.Namespace, dv.Name)
-		for _, or := range dv.OwnerReferences {
-
-			if or.Kind == "VirtualMachine" {
-				ctrl.vmQueue.Add(cacheKeyFunc(dv.Namespace, or.Name))
+		key, _ := cache.MetaNamespaceKeyFunc(dv)
+		log.Log.V(3).Infof("Processing DV %s", key)
+		// TODO come back when DV/PVC name may differ
+		for _, idx := range []string{"dv", "pvc"} {
+			keys, err := ctrl.VMInformer.GetIndexer().IndexKeys(idx, key)
+			if err != nil {
+				utilruntime.HandleError(err)
+				return
 			}
+			for _, k := range keys {
+				ctrl.vmSnapshotStatusQueue.Add(k)
+			}
+		}
+	}
+}
+
+func (ctrl *VMSnapshotController) handlePVC(obj interface{}) {
+	if unknown, ok := obj.(cache.DeletedFinalStateUnknown); ok && unknown.Obj != nil {
+		obj = unknown.Obj
+	}
+
+	if pvc, ok := obj.(*corev1.PersistentVolumeClaim); ok {
+		key, _ := cache.MetaNamespaceKeyFunc(pvc)
+		log.Log.V(3).Infof("Processing PVC %s", key)
+		keys, err := ctrl.VMInformer.GetIndexer().IndexKeys("pvc", key)
+		if err != nil {
+			utilruntime.HandleError(err)
+			return
+		}
+		for _, k := range keys {
+			ctrl.vmSnapshotStatusQueue.Add(k)
 		}
 	}
 }
