@@ -21,37 +21,37 @@ package virtwrap
 
 import (
 	"bytes"
+	"encoding/json"
 	"encoding/xml"
 	"fmt"
 	"io"
-	"net"
-	"strconv"
+	"reflect"
 	"strings"
 	"time"
 
+	cmdv1 "kubevirt.io/kubevirt/pkg/handler-launcher-com/cmd/v1"
+	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/api"
+
+	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/converter/vcpu"
+
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	utilwait "k8s.io/apimachinery/pkg/util/wait"
-	libvirt "libvirt.org/libvirt-go"
+	"libvirt.org/go/libvirt"
 
-	v1 "kubevirt.io/client-go/api/v1"
+	v1 "kubevirt.io/api/core/v1"
 	"kubevirt.io/client-go/log"
-	"kubevirt.io/kubevirt/pkg/util/net/ip"
+	virtutil "kubevirt.io/kubevirt/pkg/util"
 	cmdclient "kubevirt.io/kubevirt/pkg/virt-handler/cmd-client"
 	migrationproxy "kubevirt.io/kubevirt/pkg/virt-handler/migration-proxy"
-	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/api"
+
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/cli"
-	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/converter"
-	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/device/sriov"
+	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/device/hostdevice"
+	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/device/hostdevice/sriov"
 	domainerrors "kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/errors"
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/util"
 )
 
-// Only used for testing, migration proxy ports are 'well-known' ports and should not be randomized in production
-var osChosenMigrationProxyPort = false
-
-func setOSChosenMigrationProxyPort(val bool) {
-	osChosenMigrationProxyPort = val
-}
+const liveMigrationFailed = "Live migration failed."
 
 type migrationDisks struct {
 	shared    map[string]bool
@@ -109,7 +109,7 @@ func hotUnplugHostDevices(virConn cli.Connection, dom cli.VirDomain) error {
 		return err
 	}
 
-	eventChan := make(chan interface{}, sriov.MaxConcurrentHotPlugDevicesEvents)
+	eventChan := make(chan interface{}, hostdevice.MaxConcurrentHotPlugDevicesEvents)
 	var callback libvirt.DomainEventDeviceRemovedCallback = func(c *libvirt.Connect, d *libvirt.Domain, event *libvirt.DomainEventDeviceRemoved) {
 		eventChan <- event.DevAlias
 	}
@@ -124,23 +124,165 @@ func hotUnplugHostDevices(virConn cli.Connection, dom cli.VirDomain) error {
 	return nil
 }
 
+func generateDomainForTargetCPUSetAndTopology(vmi *v1.VirtualMachineInstance, domSpec *api.DomainSpec) (*api.Domain, error) {
+	var targetTopology cmdv1.Topology
+	targetNodeCPUSet := vmi.Status.MigrationState.TargetCPUSet
+	err := json.Unmarshal([]byte(vmi.Status.MigrationState.TargetNodeTopology), &targetTopology)
+	if err != nil {
+		return nil, err
+	}
+
+	useIOThreads := false
+	for _, diskDevice := range vmi.Spec.Domain.Devices.Disks {
+		if diskDevice.DedicatedIOThread != nil && *diskDevice.DedicatedIOThread {
+			useIOThreads = true
+			break
+		}
+	}
+	domain := api.NewMinimalDomain(vmi.Name)
+	domain.Spec = *domSpec
+	cpuTopology := vcpu.GetCPUTopology(vmi)
+	cpuCount := vcpu.CalculateRequestedVCPUs(cpuTopology)
+	domain.Spec.CPU.Topology = cpuTopology
+	domain.Spec.VCPU = &api.VCPU{
+		Placement: "static",
+		CPUs:      cpuCount,
+	}
+	err = vcpu.AdjustDomainForTopologyAndCPUSet(domain, vmi, &targetTopology, targetNodeCPUSet, useIOThreads)
+	if err != nil {
+		return nil, err
+	}
+
+	return domain, err
+}
+
+func injectNewSection(encoder *xml.Encoder, domain *api.Domain, section []string, logger *log.FilteredLogger) error {
+	// Marshalling the whole domain, even if we just need the cputune section, for indentation purposes
+	xmlstr, err := xml.MarshalIndent(domain.Spec, "", "  ")
+	if err != nil {
+		logger.Reason(err).Error("Live migration failed. Failed to get XML.")
+		return err
+	}
+	decoder := xml.NewDecoder(bytes.NewReader(xmlstr))
+	var location = make([]string, 0)
+	var newLocation []string = nil
+	injecting := false
+	for {
+		if newLocation != nil {
+			// Postpone popping end elements from `location` to ensure their removal
+			location = newLocation
+			newLocation = nil
+		}
+		token, err := decoder.RawToken()
+		if err == io.EOF {
+			break
+		}
+		if err != nil {
+			logger.Errorf("error getting token: %v\n", err)
+			return err
+		}
+
+		switch v := token.(type) {
+		case xml.StartElement:
+			location = append(location, v.Name.Local)
+
+		case xml.EndElement:
+			newLocation = location[:len(location)-1]
+		}
+
+		if len(location) >= len(section) &&
+			reflect.DeepEqual(location[:len(section)], section) {
+			injecting = true
+		} else {
+			if injecting == true {
+				// We just left the section block, we're done
+				break
+			} else {
+				// We're not in the section block yet, skipping elements
+				continue
+			}
+		}
+
+		if injecting {
+			if err := encoder.EncodeToken(xml.CopyToken(token)); err != nil {
+				logger.Reason(err).Errorf("Failed to encode token %v", token)
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
+// This function returns true for every section that should be adjusted with target data when migrating a VMI
+//   that includes dedicated CPUs
+// Strict mode only returns true if we just entered the block
+func shouldOverrideForDedicatedCPUTarget(section []string, strict bool) bool {
+	if (!strict || len(section) == 2) &&
+		len(section) >= 2 &&
+		section[0] == "domain" &&
+		section[1] == "cputune" {
+		return true
+	}
+	if (!strict || len(section) == 2) &&
+		len(section) >= 2 &&
+		section[0] == "domain" &&
+		section[1] == "numatune" {
+		return true
+	}
+	if (!strict || len(section) == 3) &&
+		len(section) >= 3 &&
+		section[0] == "domain" &&
+		section[1] == "cpu" &&
+		section[2] == "numa" {
+		return true
+	}
+
+	return false
+}
+
 // This returns domain xml without the migration metadata section, as it is only relevant to the source domain
 // Note: Unfortunately we can't just use UnMarshall + Marshall here, as that leads to unwanted XML alterations
-func migratableDomXML(dom cli.VirDomain, vmi *v1.VirtualMachineInstance) (string, error) {
+func migratableDomXML(dom cli.VirDomain, vmi *v1.VirtualMachineInstance, domSpec *api.DomainSpec) (string, error) {
+	const (
+		exactLocation = true
+		insideBlock   = false
+	)
+	var domain *api.Domain
+	var err error
+
 	xmlstr, err := dom.GetXMLDesc(libvirt.DOMAIN_XML_MIGRATABLE)
 	if err != nil {
 		log.Log.Object(vmi).Reason(err).Error("Live migration failed. Failed to get XML.")
 		return "", err
 	}
+
+	if vmi.IsCPUDedicated() {
+		// If the VMI has dedicated CPUs, we need to replace the old CPUs that were
+		// assigned in the source node with the new CPUs assigned in the target node
+		err = xml.Unmarshal([]byte(xmlstr), &domain)
+		if err != nil {
+			return "", err
+		}
+		domain, err = generateDomainForTargetCPUSetAndTopology(vmi, domSpec)
+		if err != nil {
+			return "", err
+		}
+	}
+
 	decoder := xml.NewDecoder(bytes.NewReader([]byte(xmlstr)))
 	var buf bytes.Buffer
 	encoder := xml.NewEncoder(&buf)
 
-	depth := 0
-	inMeta := false
-	inMetaKV := false
-	inMetaKVMigration := false
+	var location = make([]string, 0)
+	var newLocation []string = nil
+
 	for {
+		if newLocation != nil {
+			// Postpone popping end elements from `location` to ensure their removal
+			location = newLocation
+			newLocation = nil
+		}
 		token, err := decoder.RawToken()
 		if err == io.EOF {
 			break
@@ -152,39 +294,38 @@ func migratableDomXML(dom cli.VirDomain, vmi *v1.VirtualMachineInstance) (string
 
 		switch v := token.(type) {
 		case xml.StartElement:
-			if depth == 1 && v.Name.Local == "metadata" {
-				inMeta = true
-			} else if inMeta && depth == 2 && v.Name.Local == "kubevirt" {
-				inMetaKV = true
-			} else if inMetaKV && depth == 3 && v.Name.Local == "migration" {
-				inMetaKVMigration = true
+			location = append(location, v.Name.Local)
+
+			// If the VMI requires dedicated CPUs, we need to patch the domain with
+			// the new CPU/NUMA info calculated for the target node prior to migration
+			if vmi.IsCPUDedicated() && shouldOverrideForDedicatedCPUTarget(location, exactLocation) {
+				err = injectNewSection(encoder, domain, location, log.Log.Object(vmi))
+				if err != nil {
+					return "", err
+				}
 			}
-			depth++
 		case xml.EndElement:
-			depth--
-			if inMetaKVMigration && depth == 3 && v.Name.Local == "migration" {
-				inMetaKVMigration = false
-				continue // Skip </migration>
-			}
-			if inMetaKV && depth == 2 && v.Name.Local == "kubevirt" {
-				inMetaKV = false
-			}
-			if inMeta && depth == 1 && v.Name.Local == "metadata" {
-				inMeta = false
-			}
+			newLocation = location[:len(location)-1]
 		}
-		if inMetaKVMigration {
-			continue // We're inside metadata/kubevirt/migration, continuing to skip elements
+		if len(location) >= 4 &&
+			location[0] == "domain" &&
+			location[1] == "metadata" &&
+			location[2] == "kubevirt" &&
+			location[3] == "migration" {
+			continue // We're inside domain/metadata/kubevirt/migration, continue will skip elements
+		}
+		if vmi.IsCPUDedicated() && shouldOverrideForDedicatedCPUTarget(location, insideBlock) {
+			continue
 		}
 
 		if err := encoder.EncodeToken(xml.CopyToken(token)); err != nil {
-			log.Log.Object(vmi).Reason(err)
+			log.Log.Object(vmi).Reason(err).Errorf("Failed to encode token %v", token)
 			return "", err
 		}
 	}
 
 	if err := encoder.Flush(); err != nil {
-		log.Log.Object(vmi).Reason(err)
+		log.Log.Object(vmi).Reason(err).Error("Failed to flush XML encoder")
 		return "", err
 	}
 
@@ -268,13 +409,9 @@ func (l *LibvirtDomainManager) startMigration(vmi *v1.VirtualMachineInstance, op
 		return nil
 	}
 
-	if err := updateHostsFile(fmt.Sprintf("%s %s\n", ip.GetLoopbackAddress(), vmi.Status.MigrationState.TargetPod)); err != nil {
-		return fmt.Errorf("failed to update the hosts file: %v", err)
-	}
-
 	err = l.asyncMigrate(vmi, options)
 	if err != nil {
-		log.Log.Object(vmi).Reason(err).Error("Live migration failed.")
+		log.Log.Object(vmi).Reason(err).Error(liveMigrationFailed)
 		l.setMigrationResult(vmi, true, fmt.Sprintf("%v", err), "")
 		return err
 	}
@@ -308,7 +445,8 @@ func (l *LibvirtDomainManager) initializeMigrationMetadata(vmi *v1.VirtualMachin
 		} else {
 			// Don't allow the same migration UID to be executed twice.
 			// Migration attempts are like pods. One shot.
-			return false, fmt.Errorf("migration job already executed")
+			return false, fmt.Errorf("migration job %v already executed, finished at %v, completed: %t, failed: %t, abortStatus: %s",
+				migrationMetadata.UID, *migrationMetadata.EndTimestamp, migrationMetadata.Completed, migrationMetadata.Failed, migrationMetadata.AbortStatus)
 		}
 	}
 
@@ -654,7 +792,7 @@ func (m *migrationMonitor) startMonitor() {
 	domName := api.VMINamespaceKeyFunc(vmi)
 	dom, err := m.l.virConn.LookupDomainByName(domName)
 	if err != nil {
-		logger.Reason(err).Error("Live migration failed.")
+		logger.Reason(err).Error(liveMigrationFailed)
 		m.l.setMigrationResult(vmi, true, fmt.Sprintf("%v", err), "")
 		return
 	}
@@ -669,7 +807,7 @@ func (m *migrationMonitor) startMonitor() {
 			m.migrationFailedWithError = err
 		} else if m.migrationFailedWithError != nil {
 			logger.Info("Didn't manage to get a job status. Post the received error and finilize.")
-			logger.Reason(m.migrationFailedWithError).Error("Live migration failed")
+			logger.Reason(m.migrationFailedWithError).Error(liveMigrationFailed)
 			var abortStatus v1.MigrationAbortStatus
 			if strings.Contains(m.migrationFailedWithError.Error(), "canceled by client") {
 				abortStatus = v1.MigrationAbortSucceeded
@@ -745,70 +883,42 @@ func isBlockMigration(vmi *v1.VirtualMachineInstance) bool {
 	return (vmi.Status.MigrationMethod == v1.BlockMigration)
 }
 
-func (l *LibvirtDomainManager) generateMigrationProxies(vmi *v1.VirtualMachineInstance) []migrationproxy.MigrationProxyListener {
-	// create local migration proxies.
-	//
-	// Right now Libvirt won't let us perform a migration using a unix socket, so
-	// we have to create a local host tcp server (on port 22222) that forwards the traffic
-	// to libvirt in order to trick libvirt into doing what we want.
-	// This also creates a tcp server for each additional direct migration connections
-	// that will be proxied to the destination pod
-	migrationPortsRange := migrationproxy.GetMigrationPortsList(isBlockMigration(vmi))
-
-	proxies := []migrationproxy.MigrationProxyListener{}
-
-	loopbackAddress := ip.GetLoopbackAddress()
-	// Create a tcp server for each direct connection proxy
-	for _, port := range migrationPortsRange {
-		if osChosenMigrationProxyPort {
-			// this is only set to 0 during unit tests
-			port = 0
-		}
-		key := migrationproxy.ConstructProxyKey(string(vmi.UID), port)
-		newProxy := migrationproxy.NewTargetProxy(loopbackAddress, port, nil, nil, migrationproxy.SourceUnixFile(l.virtShareDir, key), string(vmi.UID))
-		proxies = append(proxies, newProxy)
-	}
-
-	//  proxy incoming migration requests on port 22222 to the vmi's existing libvirt connection
-	tcpBindPort := LibvirtLocalConnectionPort
-	if osChosenMigrationProxyPort {
-		// this is only set to 0 during unit tests
-		tcpBindPort = 0
-	}
-	libvirtConnectionProxy := migrationproxy.NewTargetProxy(loopbackAddress, tcpBindPort, nil, nil, migrationproxy.SourceUnixFile(l.virtShareDir, string(vmi.UID)), string(vmi.UID))
-	proxies = append(proxies, libvirtConnectionProxy)
-
-	return proxies
-}
-
-func generateMigrationParams(dom cli.VirDomain, vmi *v1.VirtualMachineInstance, options *cmdclient.MigrationOptions) (*libvirt.DomainMigrateParameters, error) {
-
-	migrURI := fmt.Sprintf("tcp://%s", ip.NormalizeIPAddress(ip.GetLoopbackAddress()))
-
-	bandwidth, err := converter.QuantityToMebiByte(options.Bandwidth)
+func generateMigrationParams(dom cli.VirDomain, vmi *v1.VirtualMachineInstance, options *cmdclient.MigrationOptions, virtShareDir string, domSpec *api.DomainSpec) (*libvirt.DomainMigrateParameters, error) {
+	bandwidth, err := vcpu.QuantityToMebiByte(options.Bandwidth)
 	if err != nil {
 		return nil, err
 	}
 
-	xmlstr, err := migratableDomXML(dom, vmi)
+	xmlstr, err := migratableDomXML(dom, vmi, domSpec)
 	if err != nil {
 		return nil, err
 	}
 
+	key := migrationproxy.ConstructProxyKey(string(vmi.UID), migrationproxy.LibvirtDirectMigrationPort)
+	migrURI := fmt.Sprintf("unix://%s", migrationproxy.SourceUnixFile(virtShareDir, key))
 	params := &libvirt.DomainMigrateParameters{
-		Bandwidth:  bandwidth, // MiB/s
-		URI:        migrURI,
-		URISet:     true,
-		DestXML:    xmlstr,
-		DestXMLSet: true,
+		URI:           migrURI,
+		URISet:        true,
+		Bandwidth:     bandwidth, // MiB/s
+		BandwidthSet:  bandwidth > 0,
+		DestXML:       xmlstr,
+		DestXMLSet:    true,
+		PersistXML:    xmlstr,
+		PersistXMLSet: true,
 	}
 
 	copyDisks := getDiskTargetsForMigration(dom, vmi)
 	if len(copyDisks) != 0 {
 		params.MigrateDisks = copyDisks
 		params.MigrateDisksSet = true
+		// add a socket for live block migration
+		key := migrationproxy.ConstructProxyKey(string(vmi.UID), migrationproxy.LibvirtBlockMigrationPort)
+		disksURI := fmt.Sprintf("unix://%s", migrationproxy.SourceUnixFile(virtShareDir, key))
+		params.DisksURI = disksURI
+		params.DisksURISet = true
 	}
 
+	log.Log.Object(vmi).Infof("generated migration parameters: %+v", params)
 	return params, nil
 }
 
@@ -817,8 +927,6 @@ func (l *LibvirtDomainManager) migrateHelper(vmi *v1.VirtualMachineInstance, opt
 	var err error
 	var params *libvirt.DomainMigrateParameters
 
-	proxies := l.generateMigrationProxies(vmi)
-
 	domName := api.VMINamespaceKeyFunc(vmi)
 	dom, err := l.virConn.LookupDomainByName(domName)
 	if err != nil {
@@ -841,8 +949,11 @@ func (l *LibvirtDomainManager) migrateHelper(vmi *v1.VirtualMachineInstance, opt
 		if err := prepareDomainForMigration(l.virConn, dom); err != nil {
 			return fmt.Errorf("error encountered during preparing domain for migration: %v", err)
 		}
-
-		params, err = generateMigrationParams(dom, vmi, options)
+		domSpec, err := l.getDomainSpec(dom)
+		if err != nil {
+			return fmt.Errorf("failed to get domain spec: %v", err)
+		}
+		params, err = generateMigrationParams(dom, vmi, options, l.virtShareDir, domSpec)
 		if err != nil {
 			return fmt.Errorf("error encountered while generating migration parameters: %v", err)
 		}
@@ -855,17 +966,14 @@ func (l *LibvirtDomainManager) migrateHelper(vmi *v1.VirtualMachineInstance, opt
 		return err
 	}
 
-	// establish all connection proxies before starting migration
-	for _, proxy := range proxies {
-		defer proxy.Stop()
-		err := proxy.Start()
-		if err != nil {
-			return fmt.Errorf("error encountered during proxy setup: %v", err)
-		}
+	// initiate the live migration
+	var dstURI string
+	if virtutil.IsNonRootVMI(vmi) {
+		dstURI = fmt.Sprintf("qemu+unix:///session?socket=%s", migrationproxy.SourceUnixFile(l.virtShareDir, string(vmi.UID)))
+	} else {
+		dstURI = fmt.Sprintf("qemu+unix:///system?socket=%s", migrationproxy.SourceUnixFile(l.virtShareDir, string(vmi.UID)))
 	}
 
-	// initiate the live migration
-	dstURI := fmt.Sprintf("qemu+tcp://%s/system", net.JoinHostPort(ip.GetLoopbackAddress(), strconv.Itoa(LibvirtLocalConnectionPort)))
 	err = dom.MigrateToURI3(dstURI, params, migrateFlags)
 	if err != nil {
 		return fmt.Errorf("error encountered during MigrateToURI3 libvirt api call: %v", err)
@@ -915,7 +1023,7 @@ func (l *LibvirtDomainManager) asyncMigrate(vmi *v1.VirtualMachineInstance, opti
 
 		err := l.migrateHelper(vmi, options)
 		if err != nil {
-			log.Log.Object(vmi).Reason(err).Error("Live migration failed.")
+			log.Log.Object(vmi).Reason(err).Error(liveMigrationFailed)
 			migrationErrorChan <- err
 			return
 		}
